@@ -1,4 +1,5 @@
-from ..exceptions import MalformedQueryException
+from .jsonb_support import supports_jsonb, _get_json_function_prefix
+from .json_path_utils import parse_json_path
 from typing import Any, Dict, List, Tuple, overload
 from typing_extensions import Literal
 
@@ -11,6 +12,8 @@ except ImportError:
 class IndexManager:
     def __init__(self, collection):
         self.collection = collection
+        # Check if JSONB is supported for this connection
+        self._jsonb_supported = supports_jsonb(collection.db)
 
     def create_index(
         self,
@@ -20,6 +23,7 @@ class IndexManager:
         unique: bool = False,
         fts: bool = False,
         tokenizer: str | None = None,
+        datetime_field: bool = False,
     ):
         """
         Create an index on the specified key(s) for this collection.
@@ -35,9 +39,15 @@ class IndexManager:
             unique: Boolean indicating whether the index should be unique.
             fts: Boolean indicating whether to create an FTS index for text search.
             tokenizer: Optional tokenizer to use for FTS index (e.g., 'icu', 'icu_th').
+            datetime_field: Boolean indicating whether this is a datetime field that requires special indexing.
         """
-        # For single key indexes, we can use SQLite's native JSON indexing
-        if isinstance(key, str):
+        # For datetime fields, use special indexing.
+        if datetime_field:
+            if isinstance(key, str):
+                self._create_datetime_index(key, unique=unique)
+            else:
+                raise ValueError("Compound datetime indexes are not supported")
+        elif isinstance(key, str):
             if fts:
                 # Create FTS index with optional tokenizer
                 self._create_fts_index(key, tokenizer)
@@ -45,12 +55,15 @@ class IndexManager:
                 # Create index name (replace dots with underscores for valid identifiers)
                 index_name = key.replace(".", "_")
 
-                # Create the index using json_extract
+                # Determine which function to use based on JSONB support
+                func_prefix = _get_json_function_prefix(self._jsonb_supported)
+
+                # Create the index using appropriate JSON/JSONB function
                 self.collection.db.execute(
                     (
                         f"CREATE {'UNIQUE ' if unique else ''}INDEX "
                         f"IF NOT EXISTS [idx_{self.collection.name}_{index_name}] "
-                        f"ON {self.collection.name}(json_extract(data, '$.{key}'))"
+                        f"ON {self.collection.name}({func_prefix}_extract(data, '$.{key}'))"
                     )
                 )
         else:
@@ -58,9 +71,12 @@ class IndexManager:
             # This is a simplified implementation - we could expand on this later
             index_name = "_".join(key).replace(".", "_")
 
-            # Create the compound index using multiple json_extract calls
+            # Determine which function to use based on JSONB support
+            func_prefix = _get_json_function_prefix(self._jsonb_supported)
+
+            # Create the compound index using multiple JSON/JSONB extract calls
             index_columns = ", ".join(
-                f"json_extract(data, '$.{k}')" for k in key
+                f"{func_prefix}_extract(data, '$.{k}')" for k in key
             )
             self.collection.db.execute(
                 (
@@ -74,66 +90,38 @@ class IndexManager:
         """
         Creates an FTS5 index on the specified field for text search.
 
+        For FTS indexes, we create both JSON and JSONB versions to ensure
+        compatibility with both types of queries.
+
         Args:
             field (str): The field to create the FTS index on.
             tokenizer (str, optional): Optional tokenizer to use for the FTS index.
-
-        Raises:
-            MalformedQueryException: If FTS5 is not available in the SQLite installation.
         """
-        # Create FTS5 virtual table
+        # Create index name (replace dots with underscores for valid identifiers)
         index_name = field.replace(".", "_")
         fts_table_name = f"{self.collection.name}_{index_name}_fts"
 
-        # Check if FTS5 is available by trying to create a simple FTS table
-        try:
-            # Try to create a temporary FTS table to check if FTS5 is available
-            self.collection.db.execute(
-                "CREATE VIRTUAL TABLE IF NOT EXISTS temp.fts_test USING fts5(test)"
-            )
-            self.collection.db.execute("DROP TABLE IF EXISTS temp.fts_test")
-        except sqlite3.OperationalError:
-            raise MalformedQueryException(
-                "FTS5 is not available in this SQLite installation"
-            )
-
-        # Create the FTS table with optional tokenizer
-        if tokenizer:
-            self.collection.db.execute(
-                f"""
-                CREATE VIRTUAL TABLE IF NOT EXISTS {fts_table_name}
-                USING fts5(
-                    content='{self.collection.name}',
-                    content_rowid='id',
-                    {index_name},
-                    tokenize='{tokenizer}'
-                )
-                """
-            )
-        else:
-            self.collection.db.execute(
-                f"""
-                CREATE VIRTUAL TABLE IF NOT EXISTS {fts_table_name}
-                USING fts5(
-                    content='{self.collection.name}',
-                    content_rowid='id',
-                    {index_name}
-                )
-                """
-            )
-
-        # Create triggers to keep the FTS index in sync with the main table
-        # Delete existing triggers if they exist
+        # Create FTS table with optional tokenizer
+        tokenizer_clause = f"TOKENIZE={tokenizer}" if tokenizer else ""
         self.collection.db.execute(
-            f"DROP TRIGGER IF EXISTS {self.collection.name}_{index_name}_fts_insert"
-        )
-        self.collection.db.execute(
-            f"DROP TRIGGER IF EXISTS {self.collection.name}_{index_name}_fts_update"
-        )
-        self.collection.db.execute(
-            f"DROP TRIGGER IF EXISTS {self.collection.name}_{index_name}_fts_delete"
+            f"""
+            CREATE VIRTUAL TABLE IF NOT EXISTS {fts_table_name}
+            USING FTS5({index_name}, content='{self.collection.name}', {tokenizer_clause})
+            """
         )
 
+        # Populate the FTS table with data from the main table
+        # For FTS, we always use json_extract to maintain compatibility
+        self.collection.db.execute(
+            f"""
+            INSERT INTO {fts_table_name}(rowid, {index_name})
+            SELECT id, lower(json_extract(data, '{parse_json_path(field)}'))
+            FROM {self.collection.name}
+            WHERE json_extract(data, '{parse_json_path(field)}') IS NOT NULL
+            """
+        )
+
+        # Create triggers to keep FTS table in sync with main table
         # Insert trigger
         self.collection.db.execute(
             f"""
@@ -141,7 +129,7 @@ class IndexManager:
             AFTER INSERT ON {self.collection.name}
             BEGIN
                 INSERT INTO {fts_table_name}(rowid, {index_name})
-                VALUES (new.id, lower(json_extract(new.data, '$.{field}')));
+                VALUES (new.id, lower(json_extract(new.data, '{parse_json_path(field)}')));
             END
             """
         )
@@ -153,9 +141,9 @@ class IndexManager:
             AFTER UPDATE ON {self.collection.name}
             BEGIN
                 INSERT INTO {fts_table_name}({fts_table_name}, rowid, {index_name})
-                VALUES ('delete', old.id, lower(json_extract(old.data, '$.{field}')));
+                VALUES ('delete', old.id, lower(json_extract(old.data, '{parse_json_path(field)}')));
                 INSERT INTO {fts_table_name}(rowid, {index_name})
-                VALUES (new.id, lower(json_extract(new.data, '$.{field}')));
+                VALUES (new.id, lower(json_extract(new.data, '{parse_json_path(field)}')));
             END
             """
         )
@@ -167,21 +155,11 @@ class IndexManager:
             AFTER DELETE ON {self.collection.name}
             BEGIN
                 INSERT INTO {fts_table_name}({fts_table_name}, rowid, {index_name})
-                VALUES ('delete', old.id, lower(json_extract(old.data, '$.{field}')));
+                VALUES ('delete', old.id, lower(json_extract(old.data, '{parse_json_path(field)}')));
             END
             """
         )
 
-        # Populate the FTS index with existing data
-        self.collection.db.execute(
-            f"""
-            INSERT INTO {fts_table_name}(rowid, {index_name})
-            SELECT id, lower(json_extract(data, '$.{field}'))
-            FROM {self.collection.name}
-            WHERE json_extract(data, '$.{field}') IS NOT NULL
-            """
-        )
-
     def create_indexes(
         self,
         indexes: List[str | List[str] | List[Tuple[str, int]] | Dict[str, Any]],
@@ -325,6 +303,10 @@ class IndexManager:
             ).fetchall()
             result = []
             for idx in indexes:
+                # Skip the automatically created _id index since it should be hidden
+                # like MongoDB's automatic _id index
+                if idx[0] == f"idx_{self.collection.name}_id":
+                    continue
                 # Extract key name from index name (idx_collection_key -> key)
                 key_name = idx[0][len(f"idx_{self.collection.name}_") :]
                 # Convert underscores back to dots for nested keys
@@ -332,12 +314,20 @@ class IndexManager:
                 result.append([key_name])
             return result
         # Return index names
-        return [
+        all_indexes = [
             idx[0]
             for idx in self.collection.db.execute(
                 cmd, (like_pattern,)
             ).fetchall()
         ]
+        # Filter out the automatically created _id index since it should be hidden
+        # like MongoDB's automatic _id index
+        filtered_indexes = [
+            idx_name
+            for idx_name in all_indexes
+            if idx_name != f"idx_{self.collection.name}_id"
+        ]
+        return filtered_indexes
 
     def drop_index(self, index: str):
         """
@@ -399,6 +389,11 @@ class IndexManager:
             ).fetchall()
 
             for idx_name, idx_sql in indexes:
+                # Skip the automatically created _id index since it should be hidden
+                # like MongoDB's automatic _id index
+                if idx_name == f"idx_{self.collection.name}_id":
+                    continue
+
                 # Parse the index information
                 index_info: Dict[str, Any] = {
                     "v": 2,  # Index version
@@ -412,11 +407,12 @@ class IndexManager:
 
                 # Try to extract key information from the SQL
                 if idx_sql:
-                    # Extract key information from json_extract expressions
+                    # Extract key information from json_extract or jsonb_extract expressions
                     import re
 
+                    # Look for both json_extract and jsonb_extract
                     json_extract_matches = re.findall(
-                        r"json_extract\(data, '(\$..*?)'\)", idx_sql
+                        r"(?:json|jsonb)_extract\(data, '(\$..*?)'\)", idx_sql
                     )
                     if json_extract_matches:
                         # Convert SQLite JSON paths back to dot notation
@@ -439,3 +435,144 @@ class IndexManager:
             pass
 
         return info
+
+    def create_search_index(
+        self,
+        key: str,
+        tokenizer: str | None = None,
+    ):
+        """
+        Create a search index on the specified key for text search functionality.
+
+        This is a convenience method that creates an FTS5 index for efficient text search.
+        It is equivalent to calling create_index(key, fts=True, tokenizer=tokenizer).
+
+        Args:
+            key: A string representing the field to index for text search.
+            tokenizer: Optional tokenizer to use for the FTS index (e.g., 'icu').
+
+        Returns:
+            The result of the index creation operation.
+        """
+        return self.create_index(key, fts=True, tokenizer=tokenizer)
+
+    def create_search_indexes(
+        self,
+        indexes: List[str],
+    ) -> List[str]:
+        """
+        Create multiple search indexes at once for text search functionality.
+
+        This is a convenience method that creates multiple FTS5 indexes for efficient text search.
+        It is equivalent to calling create_index(key, fts=True) for each key.
+
+        Args:
+            indexes: A list of strings representing the fields to index for text search.
+
+        Returns:
+            List[str]: A list of the names of the search indexes that were created.
+        """
+        created_indexes = []
+        for key in indexes:
+            # Call create_index with fts=True for each key
+            self.create_index(key, fts=True)
+            # Generate the index name the same way as in IndexManager
+            index_name = key.replace(".", "_")
+            created_indexes.append(f"idx_{self.collection.name}_{index_name}")
+        return created_indexes
+
+    def list_search_indexes(self) -> List[str]:
+        """
+        List all search indexes for the collection.
+
+        This method returns a list of all FTS5 search indexes associated with the collection.
+        Note: This implementation scans for FTS virtual tables in the database schema.
+
+        Returns:
+            List[str]: A list of search index names.
+        """
+        # Get all FTS tables from sqlite_master
+        # FTS tables have a specific naming pattern: {collection}_{field}_fts
+        fts_tables = self.collection.db.execute(
+            "SELECT name FROM sqlite_master WHERE type='table' AND name LIKE ?",
+            (f"{self.collection.name}_%_fts",),
+        ).fetchall()
+
+        # Extract the field names from the FTS table names
+        search_indexes = []
+        prefix_len = len(f"{self.collection.name}_")
+        suffix_len = len("_fts")
+
+        for (table_name,) in fts_tables:
+            # Extract field name from FTS table name
+            # Format: {collection}_{field}_fts
+            field_name = table_name[prefix_len:-suffix_len]
+            search_indexes.append(field_name)
+
+        return search_indexes
+
+    def update_search_index(self, key: str, tokenizer: str | None = None):
+        """
+        Update a search index by recreating it with potentially new options.
+
+        This method drops and recreates a search index, allowing for updates to
+        tokenizer settings or other index options.
+
+        Args:
+            key: The field name for which to update the search index.
+            tokenizer: Optional new tokenizer to use for the FTS index.
+        """
+        # Drop the existing FTS index
+        self.drop_search_index(key)
+
+        # Create a new FTS index with the updated options
+        self.create_search_index(key, tokenizer=tokenizer)
+
+    def drop_search_index(self, index: str):
+        """
+        Drop a search index from the collection.
+
+        This is a convenience method for dropping FTS5 search indexes.
+
+        Args:
+            index (str): The name of the search index to drop (field name).
+        """
+        # For FTS indexes, we need to drop the FTS virtual table and its triggers
+        # FTS tables have a specific naming pattern: {collection}_{field}_fts
+        index_name = index.replace(".", "_")
+        fts_table_name = f"{self.collection.name}_{index_name}_fts"
+
+        # Drop the FTS table
+        self.collection.db.execute(f"DROP TABLE IF EXISTS {fts_table_name}")
+
+        # Drop the triggers associated with the FTS table
+        self.collection.db.execute(
+            f"DROP TRIGGER IF EXISTS {self.collection.name}_{index_name}_fts_insert"
+        )
+        self.collection.db.execute(
+            f"DROP TRIGGER IF EXISTS {self.collection.name}_{index_name}_fts_update"
+        )
+        self.collection.db.execute(
+            f"DROP TRIGGER IF EXISTS {self.collection.name}_{index_name}_fts_delete"
+        )
+
+    def _create_datetime_index(self, key: str, unique: bool = False):
+        """Create a timezone normalized datetime index.
+
+        This method creates a specialized datetime index with datetime() for timezone
+        normalization.
+
+        Args:
+            key: The field name to create the datetime index on (e.g., 'timestamp' or 'user.created_at')
+            unique: Whether the index should be unique
+        """
+        # Generate the column name by replacing dots with underscores and appending '_utc'
+        # This is just for naming consistency
+        column_name = f"{key.replace('.', '_')}_utc"
+
+        index_sql = f"""
+        CREATE {'UNIQUE ' if unique else ''}INDEX IF NOT EXISTS
+        idx_{self.collection.name}_{column_name}
+        ON {self.collection.name}(datetime(json_extract(data, '{parse_json_path(key)}')))
+        """
+        self.collection.db.execute(index_sql)
