@@ -17,13 +17,17 @@ from neosqlite.collection.json_helpers import (
     neosqlite_json_dumps,
     neosqlite_json_loads,
 )
+from neosqlite.collection.jsonb_support import supports_jsonb
 from typing import Any, Dict, List, TYPE_CHECKING
+import importlib.util
+import json
+from copy import deepcopy
 
 if TYPE_CHECKING:
     from quez import CompressedQueue
 
-from copy import deepcopy
-import json
+# Check if quez is available
+_HAS_QUEZ = importlib.util.find_spec("quez") is not None
 
 
 class QueryEngine:
@@ -44,7 +48,11 @@ class QueryEngine:
         """
         self.collection = collection
         self.helpers = QueryHelper(collection)
-        self.sql_translator = SQLTranslator(collection.name, "data", "id")
+        # Check if JSONB is supported for this connection
+        self._jsonb_supported = supports_jsonb(collection.db)
+        self.sql_translator = SQLTranslator(
+            collection.name, "data", "id", self._jsonb_supported
+        )
 
     def insert_one(self, document: Dict[str, Any]) -> InsertOneResult:
         """
@@ -92,11 +100,41 @@ class QueryEngine:
                           including the count of matched and modified documents,
                           and the upserted ID if applicable.
         """
-        if doc := self.find_one(filter):
-            self.helpers._internal_update(doc["_id"], update, doc)
-            return UpdateResult(
-                matched_count=1, modified_count=1, upserted_id=None
-            )
+        # Apply ID type normalization to handle cases where users query 'id' with ObjectId
+        filter = self.helpers._normalize_id_query(filter)
+        # Find the document using the filter, but we need to work with integer IDs internally
+        # For internal operations, we need to retrieve the document differently to get the integer id
+        # We'll use a direct SQL query to get both the integer id and the stored _id
+        where_clause, params = self.sql_translator.translate_match(filter)
+        if where_clause:
+            # Get the integer id as well for internal operations
+            # Use the same approach as the original code considering JSONB support
+            if self.collection.query_engine._jsonb_supported:
+                cmd = f"SELECT id, _id, json(data) as data FROM {self.collection.name} {where_clause} LIMIT 1"
+            else:
+                cmd = f"SELECT id, _id, data FROM {self.collection.name} {where_clause} LIMIT 1"
+            cursor = self.collection.db.execute(cmd, params)
+            row = cursor.fetchone()
+            if row:
+                int_id, stored_id, data = row
+                # Load the document the normal way for the update processing
+                doc = self.collection._load_with_stored_id(
+                    int_id, data, stored_id
+                )
+                # Use the integer id for internal operations
+                self.helpers._internal_update(int_id, update, doc)
+                return UpdateResult(
+                    matched_count=1, modified_count=1, upserted_id=None
+                )
+        else:
+            # Fallback to find_one if translation doesn't work
+            if doc := self.find_one(filter):
+                # Get integer id by looking up the stored ObjectId
+                int_doc_id = self._get_integer_id_for_oid(doc["_id"])
+                self.helpers._internal_update(int_doc_id, update, doc)
+                return UpdateResult(
+                    matched_count=1, modified_count=1, upserted_id=None
+                )
 
         if upsert:
             # For upsert, we need to create a document that includes:
@@ -113,6 +151,24 @@ class QueryEngine:
 
         return UpdateResult(matched_count=0, modified_count=0, upserted_id=None)
 
+    def _get_integer_id_for_oid(self, oid) -> int:
+        """
+        Get the integer ID for a given ObjectId.
+        """
+        # This method should find the integer id in the database that corresponds to the ObjectId
+        cursor = self.collection.db.execute(
+            f"SELECT id FROM {self.collection.name} WHERE _id = ?",
+            (str(oid) if hasattr(oid, "__str__") else oid,),
+        )
+        row = cursor.fetchone()
+        if row:
+            return row[0]
+        # If not found, it might be that the _id is not stored in the _id column
+        # This could happen for older records
+        if isinstance(oid, int):
+            return oid
+        raise ValueError(f"Could not find integer ID for ObjectId: {oid}")
+
     def update_many(
         self,
         filter: Dict[str, Any],
@@ -131,6 +187,8 @@ class QueryEngine:
         Returns:
             UpdateResult: A result object containing information about the update operation.
         """
+        # Apply ID type normalization to handle cases where users query 'id' with ObjectId
+        filter = self.helpers._normalize_id_query(filter)
         # Try to use SQLTranslator for the WHERE clause
         where_clause, where_params = self.sql_translator.translate_match(filter)
 
@@ -150,13 +208,38 @@ class QueryEngine:
             )
 
         # Fallback for complex queries
-        docs = list(self.find(filter))
+        # Get the integer IDs for the documents to update
+        where_clause, where_params = self.sql_translator.translate_match(filter)
+        if where_clause is not None:
+            cmd = f"SELECT id FROM {self.collection.name} {where_clause}"
+            cursor = self.collection.db.execute(cmd, where_params)
+            ids = [row[0] for row in cursor.fetchall()]
+        else:
+            # If we can't translate the filter, we'll need to get all docs and filter in memory
+            docs = list(self.find(filter))
+            ids = []
+            for doc in docs:
+                int_doc_id = self._get_integer_id_for_oid(doc["_id"])
+                ids.append(int_doc_id)
+
         modified_count = 0
-        for doc in docs:
-            self.helpers._internal_update(doc["_id"], update, doc)
-            modified_count += 1
+        for int_doc_id in ids:
+            # Get the document using the integer ID for the update
+            if self.collection.query_engine._jsonb_supported:
+                cmd = f"SELECT id, _id, json(data) as data FROM {self.collection.name} WHERE id = ?"
+            else:
+                cmd = f"SELECT id, _id, data FROM {self.collection.name} WHERE id = ?"
+            cursor = self.collection.db.execute(cmd, (int_doc_id,))
+            row = cursor.fetchone()
+            if row:
+                int_id, stored_id, data = row
+                doc = self.collection._load_with_stored_id(
+                    int_id, data, stored_id
+                )
+                self.helpers._internal_update(int_doc_id, update, doc)
+                modified_count += 1
         return UpdateResult(
-            matched_count=len(docs),
+            matched_count=len(ids),
             modified_count=modified_count,
             upserted_id=None,
         )
@@ -173,9 +256,26 @@ class QueryEngine:
             DeleteResult: A result object indicating whether the deletion was
                           successful or not.
         """
-        if doc := self.find_one(filter):
-            self.helpers._internal_delete(doc["_id"])
-            return DeleteResult(deleted_count=1)
+        # Apply ID type normalization to handle cases where users query 'id' with ObjectId
+        filter = self.helpers._normalize_id_query(filter)
+        # Use direct query to get integer ID for the delete operation
+        where_clause, params = self.sql_translator.translate_match(filter)
+        if where_clause:
+            cmd = (
+                f"SELECT id FROM {self.collection.name} {where_clause} LIMIT 1"
+            )
+            cursor = self.collection.db.execute(cmd, params)
+            row = cursor.fetchone()
+            if row:
+                int_id = row[0]
+                self.helpers._internal_delete(int_id)
+                return DeleteResult(deleted_count=1)
+        else:
+            # Fallback approach
+            if doc := self.find_one(filter):
+                int_doc_id = self._get_integer_id_for_oid(doc["_id"])
+                self.helpers._internal_delete(int_doc_id)
+                return DeleteResult(deleted_count=1)
         return DeleteResult(deleted_count=0)
 
     def delete_many(self, filter: Dict[str, Any]) -> DeleteResult:
@@ -189,6 +289,8 @@ class QueryEngine:
         Returns:
             DeleteResult: A result object indicating whether the deletion was successful or not.
         """
+        # Apply ID type normalization to handle cases where users query 'id' with ObjectId
+        filter = self.helpers._normalize_id_query(filter)
         # Try to use SQLTranslator for the WHERE clause
         where_clause, params = self.sql_translator.translate_match(filter)
         if where_clause is not None:
@@ -197,17 +299,32 @@ class QueryEngine:
             return DeleteResult(deleted_count=cursor.rowcount)
 
         # Fallback for complex queries
-        docs = list(self.find(filter))
-        if not docs:
+        # Get integer IDs for the documents to delete
+        where_clause, params = self.sql_translator.translate_match(filter)
+        if where_clause is not None:
+            # Use direct SQL if possible
+            cmd = f"SELECT id FROM {self.collection.name} {where_clause}"
+            cursor = self.collection.db.execute(cmd, params)
+            ids = [row[0] for row in cursor.fetchall()]
+        else:
+            # Fallback to finding documents and getting their integer IDs
+            docs = list(self.find(filter))
+            if not docs:
+                return DeleteResult(deleted_count=0)
+            ids = []
+            for d in docs:
+                int_doc_id = self._get_integer_id_for_oid(d["_id"])
+                ids.append(int_doc_id)
+
+        if not ids:
             return DeleteResult(deleted_count=0)
 
-        ids = tuple(d["_id"] for d in docs)
         placeholders = ",".join("?" for _ in ids)
         self.collection.db.execute(
             f"DELETE FROM {self.collection.name} WHERE id IN ({placeholders})",
             ids,
         )
-        return DeleteResult(deleted_count=len(docs))
+        return DeleteResult(deleted_count=len(ids))
 
     def replace_one(
         self,
@@ -229,11 +346,31 @@ class QueryEngine:
             UpdateResult: A result object containing the number of matched and
                           modified documents and the upserted ID.
         """
-        if doc := self.find_one(filter):
-            self.helpers._internal_replace(doc["_id"], replacement)
-            return UpdateResult(
-                matched_count=1, modified_count=1, upserted_id=None
-            )
+        # Apply ID type normalization to handle cases where users query 'id' with ObjectId
+        filter = self.helpers._normalize_id_query(filter)
+        # Find the document using the filter, but get the integer ID for internal operations
+        where_clause, params = self.sql_translator.translate_match(filter)
+        if where_clause:
+            if self.collection.query_engine._jsonb_supported:
+                cmd = f"SELECT id, _id, json(data) as data FROM {self.collection.name} {where_clause} LIMIT 1"
+            else:
+                cmd = f"SELECT id, _id, data FROM {self.collection.name} {where_clause} LIMIT 1"
+            cursor = self.collection.db.execute(cmd, params)
+            row = cursor.fetchone()
+            if row:
+                int_id, stored_id, data = row
+                self.helpers._internal_replace(int_id, replacement)
+                return UpdateResult(
+                    matched_count=1, modified_count=1, upserted_id=None
+                )
+        else:
+            # Fallback approach
+            if doc := self.find_one(filter):
+                int_doc_id = self._get_integer_id_for_oid(doc["_id"])
+                self.helpers._internal_replace(int_doc_id, replacement)
+                return UpdateResult(
+                    matched_count=1, modified_count=1, upserted_id=None
+                )
 
         if upsert:
             inserted_id = self.insert_one(replacement).inserted_id
@@ -260,6 +397,9 @@ class QueryEngine:
         Returns:
             Cursor: A cursor object to iterate over the results.
         """
+        # Apply ID type normalization to handle cases where users query 'id' with ObjectId
+        if filter is not None:
+            filter = self.helpers._normalize_id_query(filter)
         return Cursor(self.collection, filter, projection, hint)
 
     def find_raw_batches(
@@ -319,6 +459,9 @@ class QueryEngine:
             Dict[str, Any]: A dictionary representing the found document,
                             or None if no document matches.
         """
+        # Apply ID type normalization to handle cases where users query 'id' with ObjectId
+        if filter is not None:
+            filter = self.helpers._normalize_id_query(filter)
         try:
             return next(iter(self.find(filter, projection, hint).limit(1)))
         except StopIteration:
@@ -338,9 +481,31 @@ class QueryEngine:
             Dict[str, Any] | None: The document that was deleted,
                                    or None if no document matches.
         """
-        if doc := self.find_one(filter):
-            self.delete_one({"_id": doc["_id"]})
-        return doc
+        # Apply ID type normalization to handle cases where users query 'id' with ObjectId
+        filter = self.helpers._normalize_id_query(filter)
+        # Find document and get its integer ID for the delete operation
+        where_clause, params = self.sql_translator.translate_match(filter)
+        if where_clause:
+            if self.collection.query_engine._jsonb_supported:
+                cmd = f"SELECT id, _id, json(data) as data FROM {self.collection.name} {where_clause} LIMIT 1"
+            else:
+                cmd = f"SELECT id, _id, data FROM {self.collection.name} {where_clause} LIMIT 1"
+            cursor = self.collection.db.execute(cmd, params)
+            row = cursor.fetchone()
+            if row:
+                int_id, stored_id, data = row
+                doc = self.collection._load_with_stored_id(
+                    int_id, data, stored_id
+                )
+                self.helpers._internal_delete(int_id)
+                return doc
+        else:
+            # Fallback approach
+            if doc := self.find_one(filter):
+                int_doc_id = self._get_integer_id_for_oid(doc["_id"])
+                self.helpers._internal_delete(int_doc_id)
+                return doc
+        return None
 
     def find_one_and_replace(
         self,
@@ -362,9 +527,31 @@ class QueryEngine:
             Dict[str, Any] | None: The original document that was replaced,
                                    or None if no document was found and replaced.
         """
-        if doc := self.find_one(filter):
-            self.replace_one({"_id": doc["_id"]}, replacement)
-        return doc
+        # Apply ID type normalization to handle cases where users query 'id' with ObjectId
+        filter = self.helpers._normalize_id_query(filter)
+        # Find document and get its integer ID for the replace operation
+        where_clause, params = self.sql_translator.translate_match(filter)
+        if where_clause:
+            if self.collection.query_engine._jsonb_supported:
+                cmd = f"SELECT id, _id, json(data) as data FROM {self.collection.name} {where_clause} LIMIT 1"
+            else:
+                cmd = f"SELECT id, _id, data FROM {self.collection.name} {where_clause} LIMIT 1"
+            cursor = self.collection.db.execute(cmd, params)
+            row = cursor.fetchone()
+            if row:
+                int_id, stored_id, data = row
+                original_doc = self.collection._load_with_stored_id(
+                    int_id, data, stored_id
+                )
+                self.helpers._internal_replace(int_id, replacement)
+                return original_doc
+        else:
+            # Fallback approach
+            if doc := self.find_one(filter):
+                int_doc_id = self._get_integer_id_for_oid(doc["_id"])
+                self.helpers._internal_replace(int_doc_id, replacement)
+                return doc
+        return None
 
     def find_one_and_update(
         self,
@@ -385,8 +572,26 @@ class QueryEngine:
             Dict[str, Any] | None: The document that was updated,
                                    or None if no document was found and updated.
         """
+        # Apply ID type normalization to handle cases where users query 'id' with ObjectId
+        filter = self.helpers._normalize_id_query(filter)
         if doc := self.find_one(filter):
-            self.update_one({"_id": doc["_id"]}, update)
+            # Get the integer id for the internal operation
+            int_doc_id = self._get_integer_id_for_oid(doc["_id"])
+            # Update by integer id to avoid conflicts
+            where_clause = "WHERE id = ?"
+            params = [int_doc_id]
+            if self.collection.query_engine._jsonb_supported:
+                cmd = f"SELECT id, _id, json(data) as data FROM {self.collection.name} {where_clause} LIMIT 1"
+            else:
+                cmd = f"SELECT id, _id, data FROM {self.collection.name} {where_clause} LIMIT 1"
+            cursor = self.collection.db.execute(cmd, params)
+            row = cursor.fetchone()
+            if row:
+                int_id, stored_id, data = row
+                doc_to_update = self.collection._load_with_stored_id(
+                    int_id, data, stored_id
+                )
+                self.helpers._internal_update(int_id, update, doc_to_update)
         return doc
 
     def count_documents(self, filter: Dict[str, Any]) -> int:
@@ -399,6 +604,8 @@ class QueryEngine:
         Returns:
             int: The number of documents matching the filter.
         """
+        # Apply ID type normalization to handle cases where users query 'id' with ObjectId
+        filter = self.helpers._normalize_id_query(filter)
         # Try to use SQLTranslator for the WHERE clause
         where_clause, params = self.sql_translator.translate_match(filter)
         if where_clause is not None:
@@ -431,6 +638,9 @@ class QueryEngine:
         Returns:
             Set[Any]: A set containing the distinct values from the specified key.
         """
+        # Apply ID type normalization to handle cases where users query 'id' with ObjectId
+        if filter is not None:
+            filter = self.helpers._normalize_id_query(filter)
         params: List[Any] = []
         where_clause = ""
 
@@ -438,8 +648,12 @@ class QueryEngine:
             # Try to use SQLTranslator for the WHERE clause
             where_clause, params = self.sql_translator.translate_match(filter)
 
+        # For distinct operations, always use json_* functions to avoid binary data issues
+        # Even if JSONB is supported, we use json_* for distinct to ensure proper text output
+        func_prefix = "json"
+
         cmd = (
-            f"SELECT DISTINCT json_extract(data, '$.{key}') "
+            f"SELECT DISTINCT {func_prefix}_extract(data, '$.{key}') "
             f"FROM {self.collection.name} {where_clause}"
         )
         cursor = self.collection.db.execute(cmd, params)
@@ -495,15 +709,10 @@ class QueryEngine:
             List[Dict[str, Any]] | CompressedQueue: The results as either a list or compressed queue.
         """
         # If memory_constrained is True and quez is available, use quez for processing
-        if memory_constrained:
-            try:
-                from quez import CompressedQueue
+        if memory_constrained and _HAS_QUEZ:
 
-                # Use quez for memory-constrained processing
-                return self._aggregate_with_quez(pipeline, batch_size)
-            except ImportError:
-                # Fall back to normal processing if quez is not available
-                pass
+            # Use quez for memory-constrained processing
+            return self._aggregate_with_quez(pipeline, batch_size)
 
         # Try existing SQL optimization first (this was previously missing)
         try:
@@ -533,7 +742,7 @@ class QueryEngine:
                                     processed_row.append(
                                         neosqlite_json_loads(value)
                                     )
-                                except:
+                                except Exception:
                                     processed_row.append(value)
                             else:
                                 processed_row.append(value)
@@ -583,8 +792,20 @@ class QueryEngine:
                 case "$sort":
                     sort_spec = stage["$sort"]
                     for key, direction in reversed(list(sort_spec.items())):
+
+                        def make_sort_key(key, dir):
+                            def sort_key(doc):
+                                val = self.collection._get_val(doc, key)
+                                # Handle None values - sort them last for ascending, first for descending
+                                if val is None:
+                                    return (0 if dir == DESCENDING else 1, None)
+                                return (0, val)
+
+                            return sort_key
+
+                        sort_key_func = make_sort_key(key, direction)
                         docs.sort(
-                            key=lambda doc: self.collection._get_val(doc, key),
+                            key=sort_key_func,
                             reverse=direction == DESCENDING,
                         )
                 case "$skip":
@@ -742,6 +963,32 @@ class QueryEngine:
                     )
         return docs
 
+    def aggregate_raw_batches(
+        self,
+        pipeline: List[Dict[str, Any]],
+        batch_size: int = 100,
+    ) -> RawBatchCursor:
+        """
+        Perform aggregation and retrieve batches of raw JSON.
+
+        Similar to the :meth:`aggregate` method but returns a
+        :class:`~neosqlite.raw_batch_cursor.RawBatchCursor`.
+
+        This method returns raw JSON batches which can be more efficient for
+        certain use cases where you want to process data in batches rather than
+        individual documents.
+
+        Args:
+            pipeline (List[Dict[str, Any]]): A list of aggregation pipeline stages to apply.
+            batch_size (int): The number of documents to include in each batch.
+
+        Returns:
+            RawBatchCursor instance.
+        """
+        return RawBatchCursor(
+            self.collection, None, None, None, batch_size, pipeline=pipeline
+        )
+
     # --- Bulk Write methods ---
     def bulk_write(
         self,
@@ -809,11 +1056,12 @@ class QueryEngine:
             CompressedQueue: A compressed queue containing the results.
         """
         try:
-            from quez import CompressedQueue
+            if _HAS_QUEZ:
+                from quez import CompressedQueue
 
-            # Create a compressed queue for results with a reasonable size
-            # Use unbounded queue to avoid blocking during population
-            result_queue = CompressedQueue()
+                # Create a compressed queue for results with a reasonable size
+                # Use unbounded queue to avoid blocking during population
+                result_queue = CompressedQueue()
 
             # Get results from normal aggregation
             results = self.aggregate(pipeline)
