@@ -8,6 +8,7 @@ temporary table generation.
 
 from typing import Any, Dict, List, Tuple
 from .cursor import DESCENDING
+from .jsonb_support import should_use_json_functions
 
 
 def _empty_result() -> Tuple[str, List[Any]]:
@@ -39,29 +40,108 @@ class SQLFieldAccessor:
     or temporary table access.
     """
 
-    def __init__(self, data_column: str = "data", id_column: str = "id"):
+    def __init__(
+        self,
+        data_column: str = "data",
+        id_column: str = "id",
+        jsonb_supported: bool = False,
+    ):
         """
         Initialize the SQLFieldAccessor with column names.
 
         Args:
             data_column: The name of the column containing JSON data (default: "data")
             id_column: The name of the column containing document IDs (default: "id")
+            jsonb_supported: Whether JSONB functions are supported (default: False)
         """
         self.data_column = data_column
         self.id_column = id_column
+        self.jsonb_supported = jsonb_supported
 
-    def get_field_access(self, field: str, context: str = "direct") -> str:
+    def _parse_json_path(self, field: str) -> str:
         """
-        Generate field access SQL based on field name and context.
+        Convert dot notation with array indexing to JSON path syntax.
+
+        Supports:
+        - Simple fields: "name" -> "$.name"
+        - Nested fields: "address.street" -> "$.address.street"
+        - Array indexing: "tags[0]" -> "$.tags[0]"
+        - Nested array access: "orders.items[2].name" -> "$.orders.items[2].name"
+        - Complex paths: "a.b[0].c[1].d" -> "$.a.b[0].c[1].d"
+
+        Args:
+            field (str): The field path in dot notation with optional array indices
+
+        Returns:
+            str: Properly formatted JSON path
+        """
+
+        # Handle special case for _id
+        if field == "_id":
+            return field
+
+        # Pattern to match field names with optional array indices
+        # This pattern matches sequences like "field", "field[0]", "field[0][1]", etc.
+
+        # Split the field path by dots while preserving array indices
+        parts = []
+        current_part = ""
+
+        i = 0
+        while i < len(field):
+            if field[i] == ".":
+                if current_part:
+                    parts.append(current_part)
+                    current_part = ""
+            elif field[i] == "[":
+                # Find the closing bracket
+                bracket_end = field.find("]", i)
+                if bracket_end != -1:
+                    # Add the array index to current part
+                    current_part += field[i : bracket_end + 1]
+                    i = bracket_end
+                else:
+                    # Malformed array index, treat as regular character
+                    current_part += field[i]
+            else:
+                current_part += field[i]
+            i += 1
+
+        # Add the last part
+        if current_part:
+            parts.append(current_part)
+
+        # Convert each part to JSON path format
+        json_parts = []
+        for part in parts:
+            # Check if part contains array indices
+            if "[" in part:
+                # Split field name from array indices
+                field_name_end = part.find("[")
+                field_name = part[:field_name_end]
+                array_indices = part[field_name_end:]
+                json_parts.append(f"{field_name}{array_indices}")
+            else:
+                json_parts.append(part)
+
+        return f"$.{'.'.join(json_parts)}"
+
+    def get_field_access(
+        self, field: str, context: str = "direct", query: dict | None = None
+    ) -> str:
+        """
+        Generate field access SQL with enhanced JSON path support.
 
         This method generates appropriate SQL expressions for accessing fields
         based on the field name and context. For the special "_id" field, it returns
-        the ID column name. For other fields, it generates a json_extract expression
-        to access the field from the JSON data column.
+        the ID column name. For other fields, it generates a json_extract or jsonb_extract
+        expression to access the field from the JSON data column, with support for complex
+        JSON paths including array indexing.
 
         Args:
             field: The field name to access
             context: The context for field access (default: "direct")
+            query: The query being processed (used to determine if text search is needed)
 
         Returns:
             SQL expression for accessing the field
@@ -70,8 +150,14 @@ class SQLFieldAccessor:
             # Special handling for _id field
             return self.id_column
         else:
-            # Use json_extract for regular fields
-            return f"json_extract({self.data_column}, '$.{field}')"
+            # Use enhanced JSON path parsing
+            json_path = self._parse_json_path(field)
+
+            # Determine whether to use json_* or jsonb_* functions
+            use_json = should_use_json_functions(query, self.jsonb_supported)
+            function_name = "json_extract" if use_json else "jsonb_extract"
+
+            return f"{function_name}({self.data_column}, '{json_path}')"
 
 
 class SQLOperatorTranslator:
@@ -126,66 +212,160 @@ class SQLOperatorTranslator:
 
             value = neosqlite_json_dumps_for_sql(value)
 
-        match operator:
-            case "$eq":
-                sql = f"{field_access} = ?"
-                params = [value]
-            case "$gt":
-                sql = f"{field_access} > ?"
-                params = [value]
-            case "$lt":
-                sql = f"{field_access} < ?"
-                params = [value]
-            case "$gte":
-                sql = f"{field_access} >= ?"
-                params = [value]
-            case "$lte":
-                sql = f"{field_access} <= ?"
-                params = [value]
-            case "$ne":
-                sql = f"{field_access} != ?"
-                params = [value]
-            case "$in":
-                if isinstance(value, (list, tuple)):
-                    placeholders = ", ".join("?" for _ in value)
-                    sql = f"{field_access} IN ({placeholders})"
-                    params = list(value)
-            case "$nin":
-                if isinstance(value, (list, tuple)):
-                    placeholders = ", ".join("?" for _ in value)
-                    sql = f"{field_access} NOT IN ({placeholders})"
-                    params = list(value)
-            case "$exists":
-                # Handle boolean value for $exists
-                if value is True:
-                    sql = f"{field_access} IS NOT NULL"
-                    params = []
-                elif value is False:
-                    sql = f"{field_access} IS NULL"
-                    params = []
-            case "$mod":
-                # Handle [divisor, remainder] array
-                if isinstance(value, (list, tuple)) and len(value) == 2:
-                    divisor, remainder = value
-                    sql = f"{field_access} % ? = ?"
-                    params = [divisor, remainder]
-            case "$size":
-                # Handle array size comparison
-                if isinstance(value, int):
-                    sql = f"json_array_length({field_access}) = ?"
+        # Check if this is a datetime comparison that should be wrapped with datetime() function
+        # This is needed when datetime fields are indexed using datetime(json_extract(...)) for timezone normalization
+        is_datetime_comparison = (
+            operator in ("$eq", "$gt", "$lt", "$gte", "$lte", "$ne")
+            and isinstance(value, str)
+            and self._is_datetime_value(value)
+        ) or (
+            operator in ("$in", "$nin")
+            and isinstance(value, (list, tuple))
+            and len(value) > 0  # Only if there are elements to check
+            and all(
+                isinstance(v, str) and self._is_datetime_value(v) for v in value
+            )
+        )
+
+        # If it's a datetime comparison, wrap both field and value with datetime() to match indexing strategy
+        if is_datetime_comparison:
+            # Wrap the field access with datetime() function for consistency with datetime indexes
+            datetime_field_access = f"datetime({field_access})"
+            match operator:
+                case "$eq":
+                    sql = f"{datetime_field_access} = datetime(?)"
                     params = [value]
-            case "$contains":
-                # Handle case-insensitive substring search
-                # Convert value to string to match Python implementation behavior
-                str_value = str(value)
-                sql = f"lower({field_access}) LIKE ?"
-                params = [f"%{str_value.lower()}%"]
-            case _:
-                # Unsupported operator
-                pass
+                case "$gt":
+                    sql = f"{datetime_field_access} > datetime(?)"
+                    params = [value]
+                case "$lt":
+                    sql = f"{datetime_field_access} < datetime(?)"
+                    params = [value]
+                case "$gte":
+                    sql = f"{datetime_field_access} >= datetime(?)"
+                    params = [value]
+                case "$lte":
+                    sql = f"{datetime_field_access} <= datetime(?)"
+                    params = [value]
+                case "$ne":
+                    sql = f"{datetime_field_access} != datetime(?)"
+                    params = [value]
+                case "$in":
+                    if isinstance(value, (list, tuple)):
+                        placeholders = ", ".join("datetime(?)" for _ in value)
+                        sql = f"{datetime_field_access} IN ({placeholders})"
+                        params = list(value)
+                case "$nin":
+                    if isinstance(value, (list, tuple)):
+                        placeholders = ", ".join("datetime(?)" for _ in value)
+                        sql = f"{datetime_field_access} NOT IN ({placeholders})"
+                        params = list(value)
+                case _:
+                    # For unsupported operators with datetime values, fall back to regular processing
+                    is_datetime_comparison = False
+                    # Use regular processing below
+        else:
+            # Regular processing for non-datetime values
+            match operator:
+                case "$eq":
+                    sql = f"{field_access} = ?"
+                    params = [value]
+                case "$gt":
+                    sql = f"{field_access} > ?"
+                    params = [value]
+                case "$lt":
+                    sql = f"{field_access} < ?"
+                    params = [value]
+                case "$gte":
+                    sql = f"{field_access} >= ?"
+                    params = [value]
+                case "$lte":
+                    sql = f"{field_access} <= ?"
+                    params = [value]
+                case "$ne":
+                    sql = f"{field_access} != ?"
+                    params = [value]
+                case "$in":
+                    if isinstance(value, (list, tuple)):
+                        # For $in operator, check if all values are datetime to decide whether to use datetime() wrapper
+                        # Only consider it as datetime if there are values AND all values are datetime strings
+                        if len(value) > 0 and all(
+                            isinstance(v, str) and self._is_datetime_value(v)
+                            for v in value
+                        ):
+                            placeholders = ", ".join(
+                                "datetime(?)" for _ in value
+                            )
+                            sql = (
+                                f"datetime({field_access}) IN ({placeholders})"
+                            )
+                            params = list(value)
+                        else:
+                            placeholders = ", ".join("?" for _ in value)
+                            sql = f"{field_access} IN ({placeholders})"
+                            params = list(value)
+                case "$nin":
+                    if isinstance(value, (list, tuple)):
+                        # For $nin operator, check if all values are datetime to decide whether to use datetime() wrapper
+                        # Only consider it as datetime if there are values AND all values are datetime strings
+                        if len(value) > 0 and all(
+                            isinstance(v, str) and self._is_datetime_value(v)
+                            for v in value
+                        ):
+                            placeholders = ", ".join(
+                                "datetime(?)" for _ in value
+                            )
+                            sql = f"datetime({field_access}) NOT IN ({placeholders})"
+                            params = list(value)
+                        else:
+                            placeholders = ", ".join("?" for _ in value)
+                            sql = f"{field_access} NOT IN ({placeholders})"
+                            params = list(value)
+                case "$exists":
+                    # Handle boolean value for $exists
+                    if value is True:
+                        sql = f"{field_access} IS NOT NULL"
+                        params = []
+                    elif value is False:
+                        sql = f"{field_access} IS NULL"
+                        params = []
+                case "$mod":
+                    # Handle [divisor, remainder] array
+                    if isinstance(value, (list, tuple)) and len(value) == 2:
+                        divisor, remainder = value
+                        sql = f"{field_access} % ? = ?"
+                        params = [divisor, remainder]
+                case "$size":
+                    # Handle array size comparison
+                    if isinstance(value, int):
+                        sql = f"json_array_length({field_access}) = ?"
+                        params = [value]
+                case "$contains":
+                    # Handle case-insensitive substring search
+                    # Convert value to string to match Python implementation behavior
+                    str_value = str(value)
+                    sql = f"lower({field_access}) LIKE ?"
+                    params = [f"%{str_value.lower()}%"]
+                case _:
+                    # Unsupported operator
+                    pass
 
         return sql, params
 
+    def _is_datetime_value(self, value: Any) -> bool:
+        """
+        Check if a value is a datetime string.
+
+        Args:
+            value: Value to check
+
+        Returns:
+            True if value is a datetime string, False otherwise
+        """
+        from .datetime_utils import is_datetime_value
+
+        return is_datetime_value(value)
+
 
 class SQLClauseBuilder:
     """
@@ -253,8 +433,9 @@ class SQLClauseBuilder:
                     condition, context, is_nested=True
                 )
                 if clause is None:
+                    # Break and return the clauses built so far
                     break
-                else:
+                elif clause:  # Only add non-empty clauses
                     # Remove "WHERE " prefix if present
                     if clause.startswith("WHERE "):
                         clause = clause[6:]
@@ -284,6 +465,7 @@ class SQLClauseBuilder:
         query: Dict[str, Any],
         context: str = "direct",
         is_nested: bool = False,
+        query_param: dict | None = None,
     ) -> Tuple[str | None, List[Any]]:
         """
         Build a WHERE clause from a MongoDB-style query.
@@ -296,6 +478,7 @@ class SQLClauseBuilder:
             query: The MongoDB-style query
             context: The context for field access (default: "direct")
             is_nested: Whether this is a nested condition within a logical operator (default: False)
+            query_param: The original query being processed (used to determine if text search is needed)
 
         Returns:
             Tuple of (WHERE clause, parameters) or (None, []) if unsupported
@@ -313,9 +496,7 @@ class SQLClauseBuilder:
                     field, value, context
                 )
                 if sql is None:
-                    return (
-                        _empty_result()
-                    )  # Unsupported condition, fallback to Python
+                    return None, []  # Unsupported condition, fallback to Python
                 else:  # Only add if not empty
                     clauses.append(sql)
                     params.extend(clause_params)
@@ -323,11 +504,12 @@ class SQLClauseBuilder:
                 # Handle $not logical operator (applies to single condition)
                 if isinstance(value, dict):
                     not_clause, not_params = self.build_where_clause(
-                        value, context, is_nested=True
+                        value, context, is_nested=True, query_param=query_param
                     )
                     if not_clause is None:
                         return (
-                            _empty_result()
+                            None,
+                            [],
                         )  # Unsupported condition, fallback to Python
                     if not_clause:
                         # Remove "WHERE " prefix if present
@@ -343,7 +525,7 @@ class SQLClauseBuilder:
                 # Regular field condition
                 # Get field access expression
                 field_access = self.field_accessor.get_field_access(
-                    field, context
+                    field, context, query_param
                 )
 
                 if isinstance(value, dict):
@@ -367,10 +549,11 @@ class SQLClauseBuilder:
         if not clauses:
             return _empty_result()
 
-        where_clause = " AND ".join(clauses)
-        # Only add "WHERE" prefix if this is not a nested condition
-        if not is_nested:
-            where_clause = "WHERE " + where_clause
+        # Build the final WHERE clause
+        where_clause = "WHERE " + " AND ".join(clauses)
+        if is_nested:
+            # Remove "WHERE " prefix for nested conditions
+            where_clause = where_clause[6:]
 
         return where_clause, params
 
@@ -446,6 +629,7 @@ class SQLTranslator:
         table_name: str | None = None,
         data_column: str = "data",
         id_column: str = "id",
+        jsonb_supported: bool = False,
     ):
         """
         Initialize the SQLTranslator with table and column names.
@@ -454,14 +638,18 @@ class SQLTranslator:
             table_name: The name of the table to query (default: "collection")
             data_column: The name of the column containing JSON data (default: "data")
             id_column: The name of the column containing document IDs (default: "id")
+            jsonb_supported: Whether JSONB functions are supported (default: False)
         """
         self.table_name = table_name or "collection"
         self.data_column = data_column
         self.id_column = id_column
+        self.jsonb_supported = jsonb_supported
 
         # Initialize components
-        self.field_accessor = SQLFieldAccessor(data_column, id_column)
-        self.operator_translator = SQLOperatorTranslator()
+        self.field_accessor = SQLFieldAccessor(
+            data_column, id_column, jsonb_supported
+        )
+        self.operator_translator = SQLOperatorTranslator(self.field_accessor)
         self.clause_builder = SQLClauseBuilder(
             self.field_accessor, self.operator_translator
         )
@@ -485,6 +673,9 @@ class SQLTranslator:
             Tuple of (WHERE clause, parameters) or (None, []) for text search
         """
         # Handle text search queries separately
+        if not isinstance(match_spec, dict):
+            return _text_search_fallback()  # Fallback for non-dict queries
+
         if "$text" in match_spec:
             return (
                 _text_search_fallback()
@@ -496,7 +687,10 @@ class SQLTranslator:
                 _text_search_fallback()
             )  # Special handling required, return None to fallback
 
-        return self.clause_builder.build_where_clause(match_spec, context)
+        # Pass the query to the clause builder so it can be used for field access decisions
+        return self.clause_builder.build_where_clause(
+            match_spec, context, query_param=match_spec
+        )
 
     def _contains_text_operator(self, query: Dict[str, Any]) -> bool:
         """
