@@ -1,5 +1,6 @@
 from __future__ import annotations
 from .collection import Collection
+from .exceptions import CollectionInvalid
 from contextlib import contextmanager
 from typing import Any, Dict, Iterator, List, Tuple
 from typing_extensions import Literal
@@ -24,9 +25,11 @@ class Connection:
             **kwargs: Keyword arguments passed to sqlite3.connect().
                      Special kwargs:
                      - tokenizers: List of tuples (name, path) for FTS5 tokenizers to load
+                     - debug: Boolean flag to enable debug printing
         """
         self._collections: Dict[str, Collection] = {}
         self._tokenizers: List[Tuple[str, str]] = kwargs.pop("tokenizers", [])
+        self.debug: bool = kwargs.pop("debug", False)
         self.connect(*args, **kwargs)
 
     def connect(self, *args: Any, **kwargs: Any):
@@ -131,6 +134,55 @@ class Connection:
         """
         self.db.execute(f"DROP TABLE IF EXISTS {name}")
 
+    def create_collection(self, name: str, **kwargs):
+        """
+        Create a new collection with specific options.
+
+        Args:
+            name (str): The name of the collection to create.
+            **kwargs: Additional options for collection creation.
+
+        Returns:
+            Collection: The newly created collection.
+
+        Raises:
+            CollectionInvalid: If a collection with the given name already exists.
+        """
+        if name in self._collections:
+            raise CollectionInvalid(f"Collection {name} already exists")
+        collection = Collection(
+            self.db, name, create=True, database=self, **kwargs
+        )
+        self._collections[name] = collection
+        return collection
+
+    def list_collection_names(self):
+        """
+        List all collection names in the database.
+
+        Returns:
+            List[str]: A list of all collection names in the database.
+        """
+        cursor = self.db.execute(
+            "SELECT name FROM sqlite_master WHERE type='table'"
+        )
+        return [row[0] for row in cursor.fetchall()]
+
+    def list_collections(self):
+        """
+        Get detailed information about collections in the database.
+
+        Returns:
+            List[Dict[str, Any]]: A list of dictionaries containing collection information.
+                                Each dictionary has 'name' and 'options' keys.
+        """
+        cursor = self.db.execute(
+            "SELECT name, sql FROM sqlite_master WHERE type='table'"
+        )
+        return [
+            {"name": row[0], "options": row[1]} for row in cursor.fetchall()
+        ]
+
     @contextmanager
     def transaction(self) -> Iterator[None]:
         """
